@@ -7,18 +7,18 @@ import matplotlib.transforms as transforms
 import matplotlib.axis as maxis
 import matplotlib.artist as artist
 from matplotlib.projections import register_projection
-from matplotlib.pyplot import rcParams,figure
+from matplotlib.pyplot import rcParams,figure,show,draw
+from matplotlib.ticker import MultipleLocator, FormatStrFormatter
 
-from thermodynamics import VirtualTemp,Latentc,SatVap,MixRatio
+from thermodynamics import VirtualTemp,Latentc,SatVap,MixRatio,GammaW,\
+	VirtualTempFromMixR
+from thermodynamics import Rs_da, Cp_da, Epsilon
 
 
 from UserDict import UserDict
 from datetime import datetime
 import os,sys
 
-R = 287.05
-Cp = 1004.
-
 class SkewXTick(maxis.XTick):
     #Copyright (c) 2008 Ryan May
     def draw(self, renderer):
@@ -172,37 +172,51 @@ class SkewXAxes(Axes):
     def other_housekeeping(self,pmin=100.,mixratio=array([])):
 	# Added by Thomas Chubb
 	self.yaxis.grid(True,ls='-',color='y',lw=0.5)
+
+	# Plot x-grid lines instead of using xaxis.grid().
+	# This is because xaxis.grid only plots skew lines 
+	# that intersect the lower x axis... a possible fix
+	# would be to use twinx() to plot upper and lower
+	# grid lines but I think that's messy too.
 	for TT in linspace(-100,100,21):
-	    self.plot([TT,TT],[1000,pmin],color='y',lw=0.5)
-	self.set_ylabel('Pressure (hPa)')
+	    self.plot([TT,TT],[1050,pmin],color='y',lw=0.5)
+
+	# self.set_ylabel('Pressure (hPa)')
 	self.set_xlabel('Temperature (C)')
 	self.set_yticks(linspace(100,1000,10))
 	self.yaxis.set_major_formatter(ScalarFormatter())
 	self.set_xlim(-40,30)
-	self.set_ylim(1013.,pmin)
+	self.set_ylim(1050.,pmin)
+	self.spines['right'].set_visible(False)
+	self.get_yaxis().set_tick_params(which="both",size=0)
+	self.get_xaxis().set_tick_params(which="both",size=0)
 
     def set_xticklocs(self,xticklocs):
 	# Added by Thomas Chubb
 	self.set_xticks(xticklocs)
 
-    def add_dry_adiabats(self,T0,P,**kwargs):
+    def add_dry_adiabats(self,T0,P,do_labels=True,**kwargs):
 	# Added by Thomas Chubb
-	P0=P[0]
-	T=array([ (st+273.15)*(P/P0)**(R/Cp)-273.15 for st in T0 ])
+	P0=1000.
+	T=array([ (st+273.15)*(P/P0)**(Rs_da/Cp_da)-273.15 for st in T0 ])
+	labelt=[ (st+273.15)*1**(Rs_da/Cp_da) for st in T0 ]
 	if kwargs.has_key('color'): 
 	    col=kwargs['color']
 	else: 
 	    col='k'
-	for tt in T:
+	for tt,ll in zip(T,labelt):
 	    self.plot(tt,P,**kwargs)
-	    if (tt[8]>-50) and (tt[8]<20):
-		self.text(tt[8],P[8]+10,'%d'%(tt[0]+273.15),fontsize=8,ha='center',va='bottom',\
-			rotation=-30,color=col,\
-			bbox={'facecolor':'w','edgecolor':'w'})
+	    if do_labels:
+		if (tt[8]>-50) and (tt[8]<20):
+		    self.text(tt[8],P[8]+10,'%d'%(ll),fontsize=8,\
+			    ha='center',va='bottom',rotation=-30,color=col,\
+			    bbox={'facecolor':'w','edgecolor':'w'})
+	return T
+    
 
-    def add_moist_adiabats(self,T0,P,**kwargs):
+    def add_moist_adiabats(self,T0,P,do_labels=True,**kwargs):
 	# Added by Thomas Chubb
-	T=array([LiftWet(st,P) for st in T0])
+	T=array([lift_wet(st,P) for st in T0])
 	if kwargs.has_key('color'): 
 	    col=kwargs['color']
 	else: 
@@ -210,11 +224,11 @@ class SkewXAxes(Axes):
 	for tt in T:
 	    self.plot(tt,P,**kwargs)
 	    # if (tt[-1]>-60) and (tt[-1]<-10):
-	    self.text(tt[-1],P[-1],'%d'%tt[0],ha='center',va='bottom',\
-		    fontsize=8,\
-		    bbox={'facecolor':'w','edgecolor':'w'},color=col)
+	    if do_labels:
+		self.text(tt[-1],P[-1],'%d'%tt[0],ha='center',va='bottom',\
+			fontsize=8, bbox={'facecolor':'w','edgecolor':'w'},color=col)
 
-    def add_mixratio_isopleths(self,w,P,**kwargs):
+    def add_mixratio_isopleths(self,w,P,do_labels=True,**kwargs):
 	# Added by Thomas Chubb
 	e=array([P*ww/(.622+ww) for ww in w])
 	T = 243.5/(17.67/log(e/6.112) - 1)
@@ -222,19 +236,18 @@ class SkewXAxes(Axes):
 	    col=kwargs['color']
 	else: 
 	    col='k'
+
 	for tt,mr in zip(T,w):
 	    self.plot(tt,P.flatten(),**kwargs)
-	    if (tt[-1]>-45) and (tt[-1]<20):
-
-		if mr*1000<1.:
-		    fmt="%4.1f"
-		else:
-		    fmt="%d"
-
-		self.text(tt[-1],P[-1],fmt%(mr*1000),\
-			color=col, fontsize=8,ha='center',va='bottom',\
-			bbox={'facecolor':'w','edgecolor':'w'})
-		# self.text(tt[0],1000,'%4.1f'%mr*1000)
+	    if do_labels:
+		if (tt[-1]>-45) and (tt[-1]<20):
+		    if mr*1000<1.:
+			fmt="%4.1f"
+		    else:
+			fmt="%d"
+		    self.text(tt[-1],P[-1],fmt%(mr*1000),\
+			    color=col, fontsize=8,ha='center',va='bottom',\
+			    bbox={'facecolor':'w','edgecolor':'w'})
 
 # Now register the projection with matplotlib so the user can select
 # it.
@@ -301,26 +314,24 @@ class Sounding(UserDict):
 	
 	'''
 
-	fig,ax,wbax=self.make_skewt_axes()
-	self.add_profile(fig,ax,wbax,color='r',lw=2,)
+	self.make_skewt_axes()
+	self.add_profile(color='r',lw=2,)
 
 	if isinstance(title, str):
-	    ax.set_title(title)
+	    self.skewxaxis.set_title(title)
 	else:
-	    ax.set_title(self['SoundingDate'])
+	    self.skewxaxis.set_title(self['SoundingDate'])
 
 	if imagename is not None:
 	    print("saving figure")
-	    fig.savefig(imagename,dpi=100)
+	    self.fig.savefig(imagename,dpi=100)
 
-	return fig,ax,wbax
+    def add_profile(self,bloc=0.5,**kwargs):
+	"""Add a new profile to the SkewT plot.
 
-    def add_profile(self,fig,ax,wbax,bloc=0.5,**kwargs):
-	"""After-market hack to add multiple profiles to the same plot.
-
-
-	It's the same routine as plot_skewt, but requires fig, ax and wbax
-	from the make_skewt_axes function as inputs.
+	Use the kwarg bloc to set the alignment of the wind barbs 
+	from the centerline (useful if plotting multplie profiles 
+	on the one axis)
 	"""
 
 	assert self['data'].has_key('pres'), "Pressure in hPa (PRES) is required!"
@@ -340,9 +351,11 @@ class Sounding(UserDict):
 	    uu=ma.zeros(p.shape)
 	    vv=ma.zeros(p.shape)
 
-	tcprof=ax.semilogy(self['data']['temp'], self['data']['pres'],zorder=5,**kwargs)
+	tcprof=self.skewxaxis.plot(self['data']['temp'], self['data']['pres'],\
+		zorder=5,**kwargs)
 	try:
-	    dpprof=ax.semilogy(self['data']['dwpt'], self['data']['pres'],zorder=5,**kwargs)
+	    dpprof=self.skewxaxis.plot(self['data']['dwpt'], self['data']['pres'],\
+		    zorder=5,**kwargs)
 	except KeyError:
 	    print "Warning: No DWPT available"
 
@@ -350,72 +363,77 @@ class Sounding(UserDict):
 
 	skip=max(1,int(nbarbs/32))
 
-	# if kwargs.has_key('color'): bcol=kwargs['color']
-	# else: bcol='k'
+	if kwargs.has_key('color'): bcol=kwargs['color']
+	else: bcol='k'
 
 	if kwargs.has_key('alpha'): balph=kwargs['alpha']
 	else: balph=1.
 
-	wbax.barbs((zeros(p.shape)+bloc)[::skip]-0.5, p[::skip], uu[::skip], vv[::skip],\
-		length=6,color='k',alpha=balph)
+	self.wbax.barbs((zeros(p.shape)+bloc)[::skip]-0.5, p[::skip],\
+		uu[::skip], vv[::skip],\
+		length=6,color=bcol,alpha=balph,lw=0.5)
 
-	ax.other_housekeeping()
-	wbax.set_xlim(-1.5,1.5)
+	self.skewxaxis.other_housekeeping()
 
 	return tcprof
 
-    def make_skewt_axes(self,pmin=100.):
-	fig = figure(figsize=(8,8))
-	fig.clf()
+    def make_skewt_axes(self,pmax=1050.,pmin=100.):
+	
+	self.fig = figure(figsize=(7,8))
+	self.fig.clf()
 	
 	rcParams.update({\
-		'axes.linewidth':1,\
-		'axes.edgecolor':'y',\
-		'ytick.color':'k',\
-		'xtick.color':'k',\
 		'font.size':10,\
-		'ytick.major.size':0,\
-		'ytick.minor.size':0})
+		})
 
-	ax=fig.add_axes([.135,.1,.815,.8], projection='skewx')
-	ax.set_yscale('log')
+	self.skewxaxis=self.fig.add_axes([.085,.1,.83,.8], projection='skewx')
+	self.skewxaxis.set_yscale('log')
 
 	xticklocs=arange(-80,45,10)
 	T0 = xticklocs
-	P0 = 1050.
 
-	P=linspace(1013.,pmin,37)
+	P=linspace(pmax,pmin,37)
 
 	w = array([0.0001,0.0004,0.001, 0.002, 0.004, 0.007, 0.01, 0.016, 0.024, 0.032])
-	ax.add_mixratio_isopleths(w,P[P>=700],color='g',ls='--',alpha=1.,lw=0.5)
-	ax.add_dry_adiabats(linspace(250,440,20)-273.15,P,color='g',ls='--',alpha=1.,lw=0.5)
-	ax.add_moist_adiabats(linspace(8,32,7),P[P>=200],color='g',ls='--',alpha=1.,lw=0.5)
+	self.skewxaxis.add_mixratio_isopleths(w,P[P>=700],color='g',ls='--',alpha=1.,lw=0.5)
+	self.skewxaxis.add_dry_adiabats(linspace(250,440,20)-273.15,P,color='g',ls='--',alpha=1.,lw=0.5)
+	self.skewxaxis.add_moist_adiabats(linspace(8,32,7),P[P>=200],color='g',ls='--',alpha=1.,lw=0.5)
+	self.skewxaxis.other_housekeeping()
 
-	wbax=fig.add_axes([0.85,0.1,0.1,0.8],sharey=ax,frameon=False)
+	self.wbax=self.fig.add_axes([0.815,0.1,0.1,0.8],sharey=self.skewxaxis,frameon=False)
+	self.wbax.xaxis.set_ticks([],[])
+	self.wbax.yaxis.grid(True,ls='-',color='y',lw=0.5)
+	for tick in self.wbax.yaxis.get_major_ticks():
+	    # tick.label1On = False
+	    pass
+	self.wbax.get_yaxis().set_tick_params(size=0,color='y')
+	self.wbax.set_xlim(-1.5,1.5)
+	self.wbax.get_yaxis().set_visible(False)
 
-	wbax.xaxis.set_ticks([],[])
-	wbax.yaxis.grid(True,ls='-',color='y',lw=0.5)
-	for tick in wbax.yaxis.get_major_ticks():
-	    tick.label1On = False
-	wbax.set_xlim(-1.5,1.5)
+	# Set up standard atmosphere height scale on 
+	# LHS of plot. It's jus
+	majorLocatorKM   = MultipleLocator(2)
+	majorLocatorKFT  = MultipleLocator(5)
+	minorLocator     = MultipleLocator(1)
 
-	ax.other_housekeeping(pmin=pmin)
+	self.kmhax=self.fig.add_axes([0.915,0.1,1e-6,0.8],frameon=True)
+	self.kmhax.xaxis.set_ticks([],[])
+	self.kmhax.spines['left'].set_color('k')
+	self.kmhax.spines['right'].set_visible(False)
+	self.kmhax.tick_params(axis='y', colors='k',labelsize=8)
+	self.kmhax.set_ylim(0,16.18)
+	self.kmhax.set_title("km/kft",fontsize=10)
+	self.kmhax.get_yaxis().set_tick_params(which="both",direction='out')
+	self.kmhax.yaxis.set_major_locator(majorLocatorKM)
+	self.kmhax.yaxis.set_minor_locator(minorLocator)
 
-	hax=fig.add_axes([0.175,0.1,0.,0.8],frameon=True)
-	hax.xaxis.set_ticks([],[])
-	hax.spines['left'].set_color('k')
-	hax.spines['right'].set_visible(False)
-	hax.tick_params(axis='y', colors='k',labelsize=8)
-	hax.set_ylim(0,16.18)
-	hax.set_title("km/kft",fontsize=10)
-
-	hay=hax.twinx()
-	hay.xaxis.set_ticks([],[])
-	hay.tick_params(axis='y', colors='k',labelsize=8)
-	hay.set_ylim(0,53.084)
-
-
-	return fig,ax,wbax
+	self.fthax=self.kmhax.twinx()
+	self.fthax.xaxis.set_ticks([],[])
+	self.fthax.tick_params(axis='y', colors='k',labelsize=8)
+	self.fthax.set_ylim(0,53.084)
+	self.fthax.get_yaxis().set_tick_params(which="both",direction='out')
+	self.fthax.yaxis.set_major_locator(majorLocatorKFT)
+	self.fthax.yaxis.set_minor_locator(minorLocator)
 
     def readfile(self,fname):
 	#--------------------------------------------------------------------
@@ -469,79 +487,85 @@ class Sounding(UserDict):
 	    self['data'][ff]=ma.masked_values(output[ff],-999.)
 
 	return None
- 
-def LiftWet(startt,pres):
+
+    def lift_parcel(self,startp,startt,startdp):
+	"""Lift a parcel to discover certain properties.
+	
+	
+	INPUTS:
+	startp:  Pressure (hPa)
+	startt:  Temperature (C)
+	startdp: Dew Point Temperature (C)
+	"""
+	from numpy import interp
+
+	assert startt>startdp,"Not a valid parcel. Check Td<Tc"
+	Pres=linspace(startp,100,100)
+
+	# Lift the dry parcel
+	T_dry=(startt+273.15)*(Pres/startp)**(Rs_da/Cp_da)-273.15 
+
+	# Mixing ratio isopleth
+	starte=SatVap(startdp)
+	startw=MixRatio(starte,startp*100)
+	e=Pres*startw/(.622+startw)
+	T_iso=243.5/(17.67/log(e/6.112)-1)
+
+	# Solve for the intersection of these lines (LCL).
+	# interp requires the x argument (argument 2)
+	# to be ascending in order!
+	P_lcl=interp(0,T_iso-T_dry,Pres)
+	T_lcl=interp(P_lcl,Pres[::-1],T_dry[::-1])
+
+	col=[.6,.6,.6]
+
+	# Plot traces below LCL
+	self.skewxaxis.plot(T_dry[Pres>=P_lcl],Pres[Pres>=P_lcl],color=col,lw=2,)
+	self.skewxaxis.plot(T_iso[Pres>=P_lcl],Pres[Pres>=P_lcl],color=col,lw=2,)
+	self.skewxaxis.plot(T_lcl,P_lcl,ls='',marker='o',mec=col,mfc=col)
+
+	# Now lift a wet parcel from the intersection point
+	preswet=linspace(P_lcl,200)
+	tempwet=lift_wet(T_lcl,preswet)
+
+	# Plot trace above LCL
+	self.skewxaxis.plot(tempwet,preswet,color=col,lw=2,)
+
+	# Add text to sounding
+	dtext ="Parcel:\n"
+	dtext+="Ps:  %6.1fhPa\n"%startp
+	dtext+="Ts:    %4.1fC\n"%startt
+	dtext+="Ds:    %4.1fC\n"%startdp
+	dtext+="Plcl:%6.1fhPa\n"%P_lcl
+	dtext+="Tlcl:  %4.1fC"%T_lcl
+
+	self.fig.text(0.1,0.895,dtext,fontname="monospace",va='top')
+
+
+def lift_wet(startt,pres):
     #--------------------------------------------------------------------
     # Lift a parcel moist adiabatically from startp to endp.
-    # Init temp is startt in C.      
+    # Init temp is startt in C, pressure levels are in hPa    
     #--------------------------------------------------------------------
 
-#     temp=zeros(pres.shape)
-#     temp[0]=startt
-#     delp=diff(pres)
-# 
-#     ii=0
-#     for pp,dp in zip(pres[1:],delp):
-# 	ii+=1
-# 	temp[ii]=temp[ii-1]-100*dp*gammaw(temp[ii-1],pp-dp/2,100)
-
     temp=startt
     t_out=zeros(pres.shape);t_out[0]=startt
     for ii in range(pres.shape[0]-1):
 	delp=pres[ii]-pres[ii+1]
- 	temp=temp-100*delp*gammaw(temp,pres[ii]-delp/2,100)
+ 	temp=temp-100*delp*GammaW(temp+273.15,(pres[ii]-delp/2)*100)
 	t_out[ii+1]=temp
 
     return t_out
 
-def gammaw(tempc,pres,rh):
-    #-----------------------------------------------------------------------
-    # Function to calculate the moist adiabatic lapse rate (deg C/Pa) based
-    # on the temperature, pressure, and rh of the environment.
-    #----------------------------------------------------------------------
-
-    tempk=tempc+273.15
-    es=SatVap(tempc)/100.
-    ws=MixRatio(es,pres)
-    w=rh*ws/100
-    tempv=VirtualTemp(tempk,w)
-    latent=Latentc(tempc)
-
-    A=1.0+latent*ws/(287.05*tempk)
-    B=1.0+0.622*latent*latent*ws/(1005*287.05*tempk*tempk)
-    Density=100*pres/(287.05*tempv)
-    Gamma=(A/B)/(1005.*Density)
-    return Gamma
-
-# def virtual(temp,mix):
-    #------------------------------------------------------------
-    # Function to return virtual temperature given temperature in 
-    # kelvin and mixing ratio in g/g.
-    #-------------------------------------------------------------
-    # return temp*(1.0+0.6*mix)
-
-# def latentc(tempc):
-    #-----------------------------------------------------------------------
-    # Function to return the latent heat of condensation in J/kg given
-    # temperature in degrees Celsius.
-    #-----------------------------------------------------------------------
-    # return 1000*(2502.2-2.43089*tempc)
-
-# def satvap2(temp):
-    #---------------------------------------------------------------
-    # Given temp in Celsius, returns saturation vapor pressure in mb
-    #---------------------------------------------------------------
-    # return 6.112*exp(17.67*temp/(temp+243.5))
-
-# def mixratio(e,p):
-    #------------------------------------------------------
-    # Given vapor pressure and pressure, return mixing ratio
-    #-------------------------------------------------------
-    # return 0.622*e/(p-e)
-
 
 if __name__=='__main__':
 
-    sounding=Sounding("../examples/94975.2013070200.txt")
-    sounding.plot_skewt("../examples/94975.2013070200.png")
+    sounding=Sounding("../examples/94975.2013070900.txt")
+    sounding.make_skewt_axes()
+    sounding.add_profile(color='r',lw=2)
+    sounding.lift_parcel(1033.,10.7,-0.9)
+
+    sounding.fig.savefig("../examples/94975.2013070900.png")
+    show()
+    
 
