@@ -22,6 +22,8 @@ Lv=2.5e6              # Latent Heat of vaporisation
 boltzmann=5.67e-8     # Stefan-Boltzmann constant
 mv=18.0153            # Mean molar mass of water vapor(g/mol)
 
+_devel="working"
+
 def Theta(tempk,pres,pref=100000.):
     """Potential Temperature
 
@@ -49,6 +51,7 @@ def Theta(tempk,pres,pref=100000.):
 
 def ThetaE():
     """Equivalent potential temperature"""
+    raise NotImplementedError
     
 
 def ThetaV(tempk,pres,e):
@@ -56,8 +59,8 @@ def ThetaV(tempk,pres,e):
     
     INPUTS
     tempk (K)
-    press (Pa)
-    e: Water vapour pressure (Pa)
+    pres (Pa)
+    e: Water vapour pressure (Pa) (Optional)
     """ 
 
     mixr=MixRatio(e,pres)
@@ -65,12 +68,12 @@ def ThetaV(tempk,pres,e):
 
     return theta*(1+mixr/Epsilon)/(1+mixr)
 
-def GammaW(tempc,pres,rh):
+def GammaW(tempk,pres,e=None):
     """Function to calculate the moist adiabatic lapse rate (deg C/Pa) based
     on the temperature, pressure, and rh of the environment.
 
     INPUTS:
-    tempc (C)
+    tempk (K)
     pres (Pa)
     RH (%)
 
@@ -78,11 +81,17 @@ def GammaW(tempc,pres,rh):
     GammaW: The moist adiabatic lapse rate (Dec C/Pa)
     """
 
-    tempk=tempc+degCtoK
+    tempc=tempk-degCtoK
     es=SatVap(tempc)
     ws=MixRatio(es,pres)
-    w=rh*ws/100
-    tempv=VirtualTemp(tempk,w)
+
+    if e is None:
+	# assume saturated
+	e=es
+
+    w=MixRatio(e,pres)
+
+    tempv=VirtualTempFromMixR(tempk,w)
     latent=Latentc(tempc)
 
     A=1.0+latent*ws/(Rs_da*tempk)
@@ -94,12 +103,28 @@ def GammaW(tempc,pres,rh):
 def Density(tempk,pres,mixr):
     """Density of moist air"""
     
-    virtualT=VirtualTemp(tempk,mixr)
+    virtualT=VirtualTempFromMixR(tempk,mixr)
     return pres/(Rs_da*virtualT)
 
 
+def VirtualTemp(tempk,pres,e):
+    """Virtual Temperature
 
-def VirtualTemp(tempk,mixr):
+    INPUTS:
+    tempk: Temperature (K)
+    e: vapour pressure (Pa)
+    p: static pressure (Pa)
+
+    OUTPUTS:
+    tempv: Virtual temperature (K)
+
+    SOURCE: hmmmm (Wikipedia)."""
+
+    tempvk=tempk/(1-(e/pres)*(1-Epsilon))
+    return tempvk
+    
+
+def VirtualTempFromMixR(tempk,mixr):
     """Virtual Temperature
 
     INPUTS:
@@ -109,9 +134,10 @@ def VirtualTemp(tempk,mixr):
     OUTPUTS:
     tempv: Virtual temperature (K)
 
-    SOURCE: hmmmm
+    SOURCE: hmmmm (Wikipedia). This is an approximation
+    based on a m
     """
-    
+
     return tempk*(1.0+0.6*mixr)
 
 def Latentc(tempc):
